#include "atoms.hh"
#include "rom.hh"

void Block::Write(unsigned offs, unsigned color_offs) const
{
    for(unsigned a=0; a<4; ++a)
        ROM.Write(offs+a, c[a].tile);
        
    unsigned palette =
        (c[0].palette<<0) |
        (c[2].palette<<2) |
        (c[1].palette<<4) |
        (c[3].palette<<6);
    
    ROM.Write(color_offs, palette);
}

void Block::Load(unsigned offs, unsigned color_offs)
{
    unsigned char palettebyte = ROM.Read(color_offs);
    for(unsigned b=0; b<4; ++b)
    {
        static const unsigned trans[4] = {0,2,1,3};
        unsigned transt = trans[b];
        
        char bit1 = 1&(palettebyte >> ((transt*2) +1));
        char bit2 = 1&(palettebyte >> ((transt*2) +0));
        
        c[b].palette = bit1*2 + bit2;
        c[b].tile    = ROM.Read(offs + b);
    }
}

void Room::Write(unsigned ptroffs, unsigned offs) const
{
    ROM.WritePointer(ptroffs, offs);
    ROM.WriteBytes(offs, 64, blocks);
}

void Room::Load(unsigned ptroffs)
{
    unsigned offs = ROM.ReadPointer(ptroffs);
    ROM.ReadData(offs, 64, blocks);
}
