#include "rom.hh"

void NopRegion(unsigned address, unsigned bytes)
{
    unsigned remain = bytes;
    unsigned char replacement[bytes];

    unsigned target = 0;
    /* If the range is small, we create JMPs. */
    while(remain > 3)
    {
        unsigned endpos = address+target+remain;
        replacement[target++] = 0x4C; // JMP
        replacement[target++] = (endpos   ) &0xFF;
        replacement[target++] = (endpos>>8) &0xFF;
        remain -= 3;
    }
    /* TODO: Mark free. */
    while(remain > 0) { replacement[target++] = 0xEA; --remain; }
    
    ROM.WriteBytes(address, bytes, replacement);
    if(bytes > 3)
        ROM.AddFreeSpace(address+3, bytes-3);
}
void AlterDelay(unsigned fivebyteaddr, unsigned char nframes)
{
    ROM.Write(fivebyteaddr+1, nframes);
}

void RemoveDelays()
{
    //ROM.Write(0xC270+1, 9); // Reload monsters even in the beginning.*** not needed anymore

    NopRegion(0xC08C, 5*4); // game or stage cleared
    NopRegion(0xC0D2, 5*1); // related to stage clearing
    NopRegion(0xC0F2, 3);   // -"-
    NopRegion(0xC102, 5*2); // after stage cleared, before end
    
    AlterDelay(0xC223, 0x01); // megaman killed
    //NopRegion(0xC223, 5*1); // megaman killed
    
    NopRegion(0xCA2D, 5*1); // boss killed
    NopRegion(0xC241, 9);    // After Megaman killed
    AlterDelay(0xD0F3, 0x02); // shutter speed
    NopRegion(0xEF84, 5*1); // before boss appears
    NopRegion(0xEFD5, 5*1); // boss pose
    NopRegion(0xF00A, 5*1); // ? 0x80 
    NopRegion(0xF012, 5*1); // ? 0x80 
    
    //AlterDelay(0xCB48, 0x10); // ? 0x80 - related to winning Wily4 battle.
    //AlterDelay(0xF203, 0x7F); // changing this number makes Wily4 battle impossible to win
    NopRegion(0xF24F, 3); // wily4 appears faster now (no palette fadings)
    
    NopRegion(0xF059, 5*1); // boss life meter filling? maybe
    AlterDelay(0x05913B, 1); // stage starts in # frames
    NopRegion(0x059290, 3); // frame wait in scrolling
    NopRegion(0x059301, 3); // frame wait in scrolling
    
    // To make scrolling faster, disable graphics updates:
    //NopRegion(0x05A044, 9); // in horizontal scrolling
    //NopRegion(0x05A082, 12); // in vertical scrolling
    
    // boss_sel:delay after shuffling numbers
    // - it's enough for it to be noticed.
    AlterDelay(0x06B945, 0x10);
    NopRegion(0x06B90C, 3); // remove the delay in displaying numbers
    
    // climbing speed
    ROM.Write(0x059829+1, 0xFE);
    ROM.Write(0x0597F7+1, 0x01);
    ROM.Write(0x059323+1, 0x01);
    
    // attempt to remove the delay in displaying boss name
    ROM.Write(0x06BB57+1, 1); // speed of boss names. Was 5, now 1.
    ROM.Write(0x06B839, 0x30); // pose length - enough for all but Cutman :)
    
    ROM.Write(0xCA4C+1, 4); // give the 'stage clear' music when boss dies (RUSH!)
    NopRegion(0xCA47,5); // ignore "stop music" when boss dies
    AlterDelay(0xCAF4, 0x10); // when Rockmonster dies (wily1)
    
    NopRegion(0xC071,5); // don't give 'stage clear' music when the item is collected
    
    // There is now no delay after killing a boss, so
    // the music that comes after it is best left supressed!
    ROM.Write(0x06B6DA+1, 0x1A); // substitute stage selection music!
    NopRegion(0x06B77A, 5); // remove boss selection music!
    //AlterDelay(0x06B775, 0x12); // change boss selection sfx
    
    NopRegion(0x06B7F6, 3); // remove the delay when boss jumps on screen
    
    AlterDelay(0x05931B, 0x08); // flashing when battle starts
    
#if 1 /* THIS SECTION ONLY WORKS IN USA MEGAMAN */
    /* We know C105 is empty for 7 bytes */
    ROM.Write(0xC105, 0xA9); //LDA
    ROM.Write(0xC106, 0x22); //sfx
    ROM.Write(0xC107, 0x20); //JSR
    ROM.Write(0xC108, 0x77);
    ROM.Write(0xC109, 0xC4); //addr
    ROM.Write(0xC10A, 0xEA); //NOP
    ROM.Write(0xC10B, 0x60); //RTS
    ROM.DelFreeSpace(0xC105, 7);
    
    ROM.Write(0xFA19, 0x20); //JSR
    ROM.Write(0xFA1A, 0x05);
    ROM.Write(0xFA1B, 0xC1); //addr
    ROM.Write(0xFA1C, 0xEA); //NOP
    ROM.Write(0xFA1D+1, 0x01); //NOP
#endif
}
